#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "lpc23xx.h"

#include "system.h"


#define USE_FIO 1

#define HOST_BAUD_U0           (57600)
#define HOST_BAUD_U1           (115200)

#define FOSC 12000000                 /* External clock input frequency (must be between 10 MHz and 25 MHz) */

#define PCLK Fpclk

/* Constants to setup the PLL. */
#define PLL_ENABLE		0x0001UL

#define PLL_MUL			( 4 - 1 )
#define PLL_DIV			0x0000UL
#define CPU_CLK_DIV		0x0003UL
#define PLL_CONNECT		0x0002UL | PLL_ENABLE
#define PLL_FEED_BYTE1	0xAA
#define PLL_FEED_BYTE2	0x55
#define PLL_LOCK		0x4000000UL
#define PLL_CONNECTED	0x2000000UL
#define OSC_ENABLE		0x20
#define OSC_STAT		0x40
#define OSC_SELECT		0x01

/* Constants to setup the MAM. */
#define MAM_TIM_3		0x03
#define MAM_MODE_FULL	0x02


/* initialize the MAM (Memory Accelerator Module) */
#if PLL_ENABLE == 1
#define CORE_FREQ (FOSC * PLL_MUL)
#else
#define CORE_FREQ (FOSC)
#endif

#if CORE_FREQ < 20000000
#define MAM_TIMING   1                /* number of CCLK to read from the FLASH */
#elif CORE_FREQ < 40000000
#define MAM_TIMING   2                /* number of CCLK to read from the FLASH */
#else
#define MAM_TIMING   3                /* number of CCLK to read from the FLASH */
#endif

/*  0=disabled,
    1=partly enabled (enabled for code prefetch, but not for data),
    2=fully enabled */
#define MAM_MODE  2


#define NOP asm volatile (" nop ")

#ifndef NULL
#define NULL    ((void *)0)
#endif

#ifndef FALSE
#define FALSE   (0)
#endif

#ifndef TRUE
#define TRUE    (1)
#endif


#endif // _CONFIG_H_

